@@ -1,4 +1,4 @@
-/* 
+/*
  *	init.c
  *	15.3.99 tn
  *
@@ -285,7 +285,9 @@ int i;
 	masterparam.old_session_size = 0;
 	masterparam.session_size = 0;
 	masterparam.charset = 0;
+	masterparam.cust_charset = g_strdup(ICONV_CUSTOM_CHARSET);
 	masterparam.outcharset = 0;
+	masterparam.cust_outcharset = g_strdup(ICONV_CUSTOM_CHARSET);
 	masterparam.redirtype = 2;
 	masterparam.lastredirpath = g_strdup("");
 	masterparam.redirtype2 = 0;
