@@ -346,7 +346,7 @@ guint i,j;
 
 	tmp[j] = '\0';
 
-	/* check if new string is not getting to long */
+	/* check if new string is not getting too long */
 	if (strlen(tmp) < MAXLINE) {
 		strcpy(str,tmp);
 	} else {
@@ -411,7 +411,7 @@ guint i,j;
 
 	tmp[j] = '\0';
 
-	/* check if new string is not getting to long */
+	/* check if new string is not getting too long */
 	if (strlen(tmp) < MAXLINE) {
 		strcpy(str,tmp);
 	} else {
@@ -468,7 +468,7 @@ guint i,j;
 
 	tmp[j] = '\0';
 
-	/* check if new string is not getting to long */
+	/* check if new string is not getting too long */
 	if (strlen(tmp) < MAXLINE) {
 		strcpy(str,tmp);
 	} else {
@@ -507,6 +507,43 @@ guint i,j;
 }
 
 
+/*
+ * check if there are illegal chars in our iconv charset string and remove them quietly
+ * for safety reasons, only letters, numbers and . : - _ / ( ) are allowed
+ * mkisofs warns itself if the entered customer i/o-charset doesn't match
+ */
+void remove_illegal_chars_iconv(gchar *str) {
+guint i, j, len;
+gchar tmp[MAXLINE];
+
+	j = 0;
+	len = strlen(str); 
+
+	if (len >= MAXLINE-1)
+		len = MAXLINE-1;
+ 
+	if (str != NULL) {
+		for (i = 0; i < len; i++) {
+			if ( ( str[i] >= 'A' && str[i] <= 'Z' )
+			  || ( str[i] >= 'a' && str[i] <= 'z' )
+			  || ( str[i] >= '0' && str[i] <= '9' )
+			  || str[i] == '.' 
+			  || str[i] == ':' 
+			  || str[i] == '-' 
+			  || str[i] == '_' 
+			  || str[i] == '/' 
+			  || str[i] == '(' 
+			  || str[i] == ')' ) {
+				tmp[j] = str[i];
+				j++;
+			}
+		}
+		tmp[j] = '\0';
+		strcpy(str, tmp);
+	}
+}
+
+
 /*
  * get a string of the form "'xxx' from 'xxx'" and extract the strings xxx
  * to artist and title. remove any escape characters.
@@ -1079,7 +1116,7 @@ gchar tmp[MAXLINE];
 
 	strip_string(str);
 
-	/* to short, do nothing */
+	/* too short, do nothing */
 	if (strlen(str) < 2) 
 		return str;
 
