@@ -70,7 +70,7 @@ gint wav_in, wavplay_quit;
 GtkWidget *wav_quit_button;
 static GtkWidget *drawa, *timer_label, *wavplay_text;
 static GtkWidget *playb, *stopb, *nextb;
-static gint playhdl, stophdl, nexthdl, prevhdl;
+static gint playhdl, stophdl, nexthdl, prevhdl, mixhdl;
 static GdkPixmap *pixmap = NULL;
 static gint wav_out;
 static gint maxseconds;
@@ -79,7 +79,10 @@ static gint maxpixels;
 static gint ptimer;
 static gint callback_tag;
 static gint error_flag;
+static gchar playing_track[MAXLINE];
+static gboolean inpause;
 
+static void read_wav_exit(GPid pid, gint status, gpointer data);
 static gboolean read_wav_out(GIOChannel *source, GIOCondition cond, gpointer data);
 static gint draw_ticks (gpointer data);
 
@@ -162,6 +165,7 @@ static void start_play(gchar *filename) {
 gchar cmd[MAXLINE];
 gchar tmptmp[MAXLINE];
 GIOChannel *gio;
+GPid pid;
 
 	/* already playing? */
 	if (wav_in != -1) {
@@ -170,7 +174,7 @@ GIOChannel *gio;
 
 	/* have we got an audiodevice? */
 	if (strcmp(setupdata.dsp_device,"") == 0) {
-		update_text(_("Warning: No DSP-device configured."));	
+		update_text(_("Warning: No Audio-device configured."));	
 		return;
 	}
 
@@ -182,13 +186,16 @@ GIOChannel *gio;
 	dolog(3, "Executing: %s\n", cmd);
 
 	/* start child and get new fds */
-	full_dpl_pipe_shell(&wav_out,&wav_in,cmd);
+	pid = (GPid)full_dpl_pipe_shell(&wav_out,&wav_in,cmd);
 
 	/* catch output of child */
         gio = g_io_channel_unix_new(wav_out);
         g_io_channel_set_encoding(gio, NULL, NULL);
-        callback_tag = g_io_add_watch(gio, G_IO_IN | G_IO_HUP,
+        callback_tag = g_io_add_watch(gio, G_IO_IN,
                 read_wav_out, NULL);
+
+	/* check termination of child */
+	g_child_watch_add(pid, read_wav_exit, (gpointer)gio);	
 }
 
 
@@ -378,9 +385,21 @@ gchar tmp[MAXLINE];
 		gtk_clist_select_row(play_clist,0,0);
 		get_sel_trackname(tmp);
 	}
-	
+
+    /* did we change the track selection? */
+    if (strcmp(tmp, playing_track) != 0) {
+        /* quit playing current track */
+	    gtk_button_clicked(GTK_BUTTON(wav_quit_button));
+
+	    /* now wait until we are really finished */
+	    while (wav_in != -1) {
+		    wait_and_process_events();
+	    }
+    } 
+    
 	/* got a valid selection? start play it */
 	if (strcmp(tmp,"") != 0) {
+		strncpy(playing_track, tmp, MAXLINE);
 		start_play(tmp);
 	}
 
@@ -400,6 +419,7 @@ gchar tmp[MAXLINE];
 	if (write(wav_in, "play\n", 5) != 5) {
 		g_warning("write error to pipe\n");
 	}
+    inpause = FALSE;
 }
 
 static void dostop(GtkWidget *widget, gpointer data) {
@@ -417,6 +437,7 @@ static void dostop(GtkWidget *widget, gpointer data) {
 	if (write(wav_in, "stop\n", 5) != 5) {
 		g_warning("write error to pipe\n");
 	}
+    inpause = FALSE;
 }
 
 static void dopause(GtkWidget *widget, gpointer data) {
@@ -424,10 +445,17 @@ static void dopause(GtkWidget *widget, gpointer data) {
 	/* pipes not yet set up */
 	if (wav_in == -1) return;
 
+    /* already paused? Toggle play */
+    if (inpause) {
+	    gtk_button_clicked(GTK_BUTTON(playb));
+        return;
+    }
+
 	dodebug(8,"wavplay: sending command pause\n");
 	if (write(wav_in, "pause\n", 6) != 6) {
 		g_warning("write error to pipe\n");
 	}
+    inpause = TRUE;
 }
 
 static void doquit(GtkWidget *widget, gpointer data) {
@@ -592,6 +620,23 @@ gint i;
 	return(TRUE);
 }
 
+/*
+ * called when the wavplay child terminated
+ */
+static void read_wav_exit(GPid pid, gint status, gpointer data) {
+
+	dodebug(10, "wavplay terminated with status: %d\n", status);
+
+	/* cleanup */ 
+	g_io_channel_unref((GIOChannel *)data);
+
+    strcpy(playing_track, "");
+	gtk_label_set_text(GTK_LABEL(timer_label),"0:00");	
+
+	close(wav_out);
+	close(wav_in);
+	stop_play();
+}
 
 /*
  * called when new data from stdout of child is available
@@ -602,18 +647,7 @@ gchar line[MAXLINE];
 gchar cmd[MAXLINE];
 gchar tmp[MAXLINE];
 gint tsec;
-gint ret;
 
-    	if (cond == G_IO_HUP) {
-                g_io_channel_unref(source);
-
-		wait(&ret);
-		close(wav_out);
-		close(wav_in);
-		stop_play();
-
-		return (FALSE);
-	}
 	n = read(wav_out, line, MAXLINE);
 	if (n < 0) {
 		g_warning("read error from pipe\n");
@@ -626,7 +660,7 @@ gint ret;
 	/* warning found - device busy? */
 	if (strncmp(line,"\n**",3) == 0) {
 		error_flag = 1;
-		update_text(_("Error accessing DSP-device.\nBusy by another program?"));
+		update_text(_("Error accessing Audio-device.\nBusy by another program?"));
 		return (TRUE);
 	}
 
@@ -857,11 +891,13 @@ GtkWidget *slider;
 	gtk_scale_set_draw_value(GTK_SCALE(slider),FALSE);
 	gtk_table_attach_defaults(GTK_TABLE(tbl2),slider,15,20,0,1);
 	gtk_widget_show(slider);	
-	gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
+	mixhdl = gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
 		GTK_SIGNAL_FUNC(vol_changed), NULL);
 	define_tooltip(slider, _("Change the volume of playback."));
 
+	gtk_signal_handler_block(GTK_OBJECT(adj), mixhdl);
 	gtk_adjustment_set_value(GTK_ADJUSTMENT(adj), (gfloat) def_mix_val);
+	gtk_signal_handler_unblock(GTK_OBJECT(adj), mixhdl);
 }
 
 
@@ -878,6 +914,8 @@ gint mixval;
 	wav_in = -1; wav_out = -1;
 	error_flag = 0;
 	wavplay_quit = 0;
+    inpause = FALSE;
+    strcpy(playing_track, "");
 
 	f1 = gtk_frame_new(NULL);
 	gtk_frame_set_shadow_type(GTK_FRAME(f1),GTK_SHADOW_IN);
