@@ -6,6 +6,7 @@
  *
  *	works fine on little and big endian machines.
  *	30.05.99 tn
+ *  07.11.18 tn - add ALSA support for linux
  *
  *
  *  Copyright (C) 1995, 1996, 1997 Free Software Foundation, Inc.
@@ -43,7 +44,10 @@
 # include <getopt.h>
 #endif
 #include <sys/stat.h>
-#if defined(linux) || defined(__FreeBSD__)
+#if defined(linux)
+# include <alsa/asoundlib.h>
+#endif
+#if defined(__FreeBSD__)
 # include <sys/soundcard.h>
 # include <sys/ioctl.h>
 #endif
@@ -76,13 +80,18 @@
 
 static off_t waveBase ;
 static gint abuf_size;
+#if defined(linux) 
+static gint alsa_frame_size;
+#endif
 static guchar *audiobuf;
 
 gint read_line(gint fd, gchar *ptr, gint maxlen);
 off_t is_std_wav_file(int f, off_t *offset);
 
 
-#if defined(linux) || defined(__FreeBSD__)
+#if defined(linux) 
+#define DEFAULT_AUDIO_DEVICE "default"
+#elif defined(__FreeBSD__)
 #define DEFAULT_AUDIO_DEVICE "/dev/dsp"
 #elif defined (aix)
 #define DEFAULT_AUDIO_DEVICE ""
@@ -95,7 +104,89 @@ off_t is_std_wav_file(int f, off_t *offset);
 #define DEFAULT_BUFFER_SIZE 22050 
 
 
-#if defined(linux) || defined(__FreeBSD__)
+#if defined(linux) 
+
+/*
+ * open the alsa-device of linux and set cd-quality
+ */
+snd_pcm_t *open_alsa_audio(gchar *dev) {
+snd_pcm_t *playback_handle;
+snd_pcm_hw_params_t *hw_params;
+snd_pcm_uframes_t buffer_size, periods;
+gint err;
+
+	if ((err = snd_pcm_open (&playback_handle, dev, SND_PCM_STREAM_PLAYBACK, 0)) < 0) {
+			g_warning("cannot open audio device %s (%s)\n", dev, snd_strerror (err));
+			return(NULL);
+	}
+	if ((err = snd_pcm_hw_params_malloc (&hw_params)) < 0) {
+			g_warning("cannot allocate hardware parameter structure (%s)\n",
+				 snd_strerror (err));
+			return(NULL);
+	}
+				 
+	if ((err = snd_pcm_hw_params_any (playback_handle, hw_params)) < 0) {
+			g_warning("cannot initialize hardware parameter structure (%s)\n",
+				 snd_strerror (err));
+			return(NULL);
+	}
+	
+	if ((err = snd_pcm_hw_params_set_access (playback_handle, hw_params, SND_PCM_ACCESS_RW_INTERLEAVED)) < 0) {
+			g_warning("cannot set access type (%s)\n", snd_strerror (err));
+			return(NULL);
+	}
+	
+	if ((err = snd_pcm_hw_params_set_format (playback_handle, hw_params, SND_PCM_FORMAT_S16_LE)) < 0) {
+			g_warning("cannot set sample format (%s)\n", snd_strerror (err));
+			return(NULL);
+	}
+	
+	if ((err = snd_pcm_hw_params_set_rate (playback_handle, hw_params, 44100, 0)) < 0) {
+			g_warning("cannot set sample rate (%s)\n", snd_strerror (err));
+			return(NULL);
+	}
+	
+	if ((err = snd_pcm_hw_params_set_channels (playback_handle, hw_params, 2)) < 0) {
+			g_warning("cannot set channel count (%s)\n", snd_strerror (err));
+			return(NULL);
+	}
+
+    periods = 4;
+	if ((err = snd_pcm_hw_params_set_periods (playback_handle, hw_params, periods, 0)) < 0) {
+			g_warning("cannot set periode size (%s)\n", snd_strerror (err));
+			return(NULL);
+	}
+
+    buffer_size = CDDAFRAME * periods;
+	if ((err = snd_pcm_hw_params_set_buffer_size_near (playback_handle, hw_params, &buffer_size)) < 0) {
+			g_warning("cannot set buffer size (%s)\n", snd_strerror (err));
+			return(NULL);
+	}
+	
+	if ((err = snd_pcm_hw_params (playback_handle, hw_params)) < 0) {
+			g_warning("cannot set parameters (%s)\n", snd_strerror (err));
+			return(NULL);
+	}
+	
+    /* determine real buffer size */
+    snd_pcm_hw_params_get_buffer_size(hw_params, &buffer_size);
+    alsa_frame_size = snd_pcm_format_size(SND_PCM_FORMAT_S16_LE, 2);
+    abuf_size = (gint)buffer_size * alsa_frame_size;
+
+	snd_pcm_hw_params_free (hw_params);
+
+	if ((err = snd_pcm_prepare (playback_handle)) < 0) {
+			g_warning("cannot prepare audio interface for use (%s)\n",
+				 snd_strerror (err));
+			return(NULL);
+		}
+
+	return (playback_handle);
+}
+
+#endif
+
+#if defined(__FreeBSD__)
 
 /*
  * open the sound-device of linux and set cd-quality
@@ -528,7 +619,6 @@ gint main(gint argc, gchar **argv) {
 gint c;
 gchar audio_dev[MAXLINE];
 gchar wavname[MAXLINE];
-gint audio = -1;
 gint fd;
 gint l;
 gint guimode = 0;
@@ -541,9 +631,15 @@ off_t totalbytes;
 gint min,sec;
 gchar keybuffer[MAXLINE];
 #if !(defined(linux))
+ gint audio = -1;
  gint ii;
  guchar tmpswap;
 #endif
+#if defined(linux)
+ snd_pcm_t *playback_handle;
+ snd_pcm_sframes_t frames, wroteframes;
+#endif
+
 	/* if no device given, set default device */
 	strcpy(audio_dev,DEFAULT_AUDIO_DEVICE);
 	strcpy(wavname,"");
@@ -579,7 +675,10 @@ gchar keybuffer[MAXLINE];
 
 	/* the open sets also the global abuf_size-variable */
 
-#if defined(linux) || defined(__FreeBSD__)
+#if defined(linux) 
+	playback_handle = open_alsa_audio(audio_dev);
+#endif
+#if defined(__FreeBSD__)
 	audio = open_linux_audio(audio_dev);
 #endif
 #if defined(sun) || defined(__OpenBSD__)
@@ -595,10 +694,18 @@ gchar keybuffer[MAXLINE];
 	audio = open_irix_audio(audio_dev);
 #endif
 
+#if defined(linux)
+	if (playback_handle == NULL) {
+		g_warning("Can't init sound-system\n");
+		exit(1); 
+	}
+#endif
+#if !(defined(linux))
 	if (audio < 0) {
 		g_warning("Can't init sound-system\n");
 		exit(1); 
 	}
+#endif
 
 	if ((audiobuf = (guchar *)g_new(guchar *,abuf_size)) == NULL) {
 		g_warning("Unable to allocate audio-buffer\n");
@@ -647,6 +754,9 @@ gchar keybuffer[MAXLINE];
 			/* stop command */
 			if (g_ascii_strncasecmp(keybuffer,"stop",4) == 0) {
 				lseek(fd, waveBase, SEEK_SET);
+#if defined(linux)
+                snd_pcm_drain(playback_handle);
+#endif
 				bytessofar = 0;
 				tick = 0;
 				doplay = 0;
@@ -656,14 +766,19 @@ gchar keybuffer[MAXLINE];
 
 			/* play command */
 			if (g_ascii_strncasecmp(keybuffer,"play",4) == 0) {
+#if defined(linux)
+                snd_pcm_prepare(playback_handle);
+#endif
 				doplay = 1;
 			}	
 
 			/* pause command */
 			if (g_ascii_strncasecmp(keybuffer,"pause",5) == 0) {
-				doplay=1-doplay;
-				g_print("%s%d\n",doplay?"play":"stop",tick);
-				fflush(stdout);
+                if (doplay) {
+				    doplay = 0;
+				    g_print("%s%d\n","stop",tick);
+				    fflush(stdout);
+                }
 			}
 	
 			/* setxxx command */
@@ -720,6 +835,13 @@ gchar keybuffer[MAXLINE];
 
 #if defined(__sgi)
 				write_irix_audio(audiobuf, l);
+#elif defined(linux)
+                frames = (snd_pcm_sframes_t) (l/alsa_frame_size);
+				wroteframes = snd_pcm_writei (playback_handle, audiobuf, frames);
+				if (wroteframes != frames) {
+					g_print("write error to audio-device (wrote %d/%d)\n", (gint)wroteframes, (gint)frames);
+					exit(-1);
+				}
 #else
 				if (write(audio, audiobuf, l) != l) {
 					g_print("write error to audio-device\n");
@@ -759,6 +881,8 @@ gchar keybuffer[MAXLINE];
 
 #if defined(__sgi)
 	close_irix_audio();
+#elif defined(linux)
+	snd_pcm_close (playback_handle);
 #else
 	close(audio);
 #endif
