@@ -903,10 +903,10 @@ gint i,len,len2;
 			len = strlen(img_ext[i]);
 			len2 = strlen(ent->d_name);
 
-			/* skip to short filenames */
+			/* skip too short filenames */
 			if (len2 < len) continue;
 
-			if (strncmp((ent->d_name)+len2-len,img_ext[i],len) == 0) {
+			if (strncasecmp((ent->d_name)+len2-len,img_ext[i],len) == 0) {
 				/* we found a match */
 				analyze_imgfile(path,ent->d_name,retlist);			
 			}
